#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Conversion of the nonlagma indicator from MetaTrader
    /// </summary>
    [Description("Conversion of the nonlagma v4 indicator from MetaTrader")]
    public class NonLagDot : Indicator
    {
        #region Variables
        // Wizard generated variables
		private int period = 20; // Default setting for period
		private double filter = 0;
		private double splitTick = 1;
		private int cycle = 4;		
		private double coeff = 3;
		private double shift = 0.0;
        // User defined variables (add any user defined variables below)
		private DataSeries serie;
//		private DataSeries trend;
		private DataSeries sTrend;
		private int phase = 0;
		private double len = 0;		
		//private double deviation = 0.0;
		private double calc_Coeff = 0.0;
		private bool bInit = true;	
		private Color colorUp = Color.DodgerBlue;
		private Color colorDn = Color.Red;
		private Color colorNeutral = Color.Orange;
		private Color lastColor = Color.Orange;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {	
            Add(new Plot(lastColor, PlotStyle.Dot, "Trend"));
			
            CalculateOnBarClose	= true;
            Overlay				= true;
            PriceTypeSupported	= false;
			
			serie = new DataSeries(this);
			sTrend = new DataSeries(this);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {            
			if (bInit)
			{
				calc_Coeff = coeff * Math.PI;
				phase = period - 1;
				len = period * cycle + phase;			
				bInit = false;
			}
			
			double g, t, beta, alpha, Sum, Weight;
			t = 0; Sum = 0; Weight = 0;												
			
			if (CurrentBar < len) return;
			
			for (int i = 0; i <= len-1; i++)
			{	
				g = 1 / (calc_Coeff * t + 1);				
				if (t <= 0.5) g = 1;
				beta = Math.Cos(Math.PI * t);
				alpha = g * beta;
				Sum += alpha*Close[i];
				Weight += alpha;
				if (i <= phase-1) t = 1.0 * i/(phase-i);
				else t = 1.0 + (i-phase+1) * (2.0*cycle-1.0) / (cycle*period-1.0);
			}
			
			serie[0] = serie[1];
			
			if (Weight > 0)
				serie.Set(0, (1.0 + shift/100) * Sum/Weight);
			
			if (filter > 0)
			{
				if (Math.Abs(serie[0] - serie[1]) < filter * TickSize / splitTick)
					serie.Set(0, serie[1]);
			}
			
			Values[0].Set(0, serie[0]);
			sTrend[0] = sTrend[1];
			
			if (serie[0] - serie[1] > filter * TickSize / splitTick)			
				sTrend.Set(0, 1);				            
			else if (serie[1] - serie[0] > filter * TickSize / splitTick) 						
				sTrend.Set(0, -1);
			else 
				sTrend.Set(0, 0);
			
			if (sTrend[0] > 0 && sTrend[1] > 0)
			{
				PlotColors[0][0] = colorUp;		
				lastColor = colorUp;
			}
			else if (sTrend[0] < 0 && sTrend[1] < 0)
			{
				PlotColors[0][0] = colorDn;
				lastColor = colorDn;
			}	
			else
			{
				PlotColors[0][0] = colorNeutral;
				lastColor = colorNeutral;
			}
        }

        #region Properties        

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Trend
        {
            get { return Values[0]; }
        }

		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries STrend
        {
            get { Update(); return sTrend; }
        }		
		
        [Description("Period")]
        [Category("Parameters")]
		[Gui.Design.DisplayName ("1 Period")]
        public int Period
        {
            get { return period; }
            set { period = Math.Max(1, value); }
        }
		
		[Description("Cycle")]
        [Category("Parameters")]
		[Gui.Design.DisplayName ("2 Cycle")]
        public int Cycle
        {
            get { return cycle; }
            set { cycle = Math.Max(1, value); }
        }
		
		[Description("Coefficient")]
        [Category("Parameters")]
		[Gui.Design.DisplayName ("3 Coeff")]
        public double Coeff
        {
            get { return coeff; }
            set { coeff = Math.Max(1, value); }
        }
		
		[Description("Filter")]
        [Category("Parameters")]
		[Gui.Design.DisplayName ("4 Filter")]
        public double Filter
        {
            get { return filter; }
            set { filter = Math.Max(0, value); }
        }
		
		[Description("TickSize Divisor")]
        [Category("Parameters")]
		[Gui.Design.DisplayName ("5 SplitTick")]
        public double SplitTick
        {
            get { return splitTick; }
            set { splitTick = Math.Max(1, value); }
        }
		
		[Description("Displacement")]
        [Category("Parameters")]
		[Gui.Design.DisplayName ("6 Shift")]
        public double  Shift
        {
            get { return shift; }
            set { shift = Math.Max(0, value); }
        }
		
        #endregion
    }
}


#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private NonLagDot[] cacheNonLagDot = null;

        private static NonLagDot checkNonLagDot = new NonLagDot();

        /// <summary>
        /// Conversion of the nonlagma v4 indicator from MetaTrader
        /// </summary>
        /// <returns></returns>
        public NonLagDot NonLagDot(double coeff, int cycle, double filter, int period, double shift, double splitTick)
        {
            return NonLagDot(Input, coeff, cycle, filter, period, shift, splitTick);
        }

        /// <summary>
        /// Conversion of the nonlagma v4 indicator from MetaTrader
        /// </summary>
        /// <returns></returns>
        public NonLagDot NonLagDot(Data.IDataSeries input, double coeff, int cycle, double filter, int period, double shift, double splitTick)
        {
            if (cacheNonLagDot != null)
                for (int idx = 0; idx < cacheNonLagDot.Length; idx++)
                    if (Math.Abs(cacheNonLagDot[idx].Coeff - coeff) <= double.Epsilon && cacheNonLagDot[idx].Cycle == cycle && Math.Abs(cacheNonLagDot[idx].Filter - filter) <= double.Epsilon && cacheNonLagDot[idx].Period == period && Math.Abs(cacheNonLagDot[idx].Shift - shift) <= double.Epsilon && Math.Abs(cacheNonLagDot[idx].SplitTick - splitTick) <= double.Epsilon && cacheNonLagDot[idx].EqualsInput(input))
                        return cacheNonLagDot[idx];

            lock (checkNonLagDot)
            {
                checkNonLagDot.Coeff = coeff;
                coeff = checkNonLagDot.Coeff;
                checkNonLagDot.Cycle = cycle;
                cycle = checkNonLagDot.Cycle;
                checkNonLagDot.Filter = filter;
                filter = checkNonLagDot.Filter;
                checkNonLagDot.Period = period;
                period = checkNonLagDot.Period;
                checkNonLagDot.Shift = shift;
                shift = checkNonLagDot.Shift;
                checkNonLagDot.SplitTick = splitTick;
                splitTick = checkNonLagDot.SplitTick;

                if (cacheNonLagDot != null)
                    for (int idx = 0; idx < cacheNonLagDot.Length; idx++)
                        if (Math.Abs(cacheNonLagDot[idx].Coeff - coeff) <= double.Epsilon && cacheNonLagDot[idx].Cycle == cycle && Math.Abs(cacheNonLagDot[idx].Filter - filter) <= double.Epsilon && cacheNonLagDot[idx].Period == period && Math.Abs(cacheNonLagDot[idx].Shift - shift) <= double.Epsilon && Math.Abs(cacheNonLagDot[idx].SplitTick - splitTick) <= double.Epsilon && cacheNonLagDot[idx].EqualsInput(input))
                            return cacheNonLagDot[idx];

                NonLagDot indicator = new NonLagDot();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Coeff = coeff;
                indicator.Cycle = cycle;
                indicator.Filter = filter;
                indicator.Period = period;
                indicator.Shift = shift;
                indicator.SplitTick = splitTick;
                Indicators.Add(indicator);
                indicator.SetUp();

                NonLagDot[] tmp = new NonLagDot[cacheNonLagDot == null ? 1 : cacheNonLagDot.Length + 1];
                if (cacheNonLagDot != null)
                    cacheNonLagDot.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheNonLagDot = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Conversion of the nonlagma v4 indicator from MetaTrader
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.NonLagDot NonLagDot(double coeff, int cycle, double filter, int period, double shift, double splitTick)
        {
            return _indicator.NonLagDot(Input, coeff, cycle, filter, period, shift, splitTick);
        }

        /// <summary>
        /// Conversion of the nonlagma v4 indicator from MetaTrader
        /// </summary>
        /// <returns></returns>
        public Indicator.NonLagDot NonLagDot(Data.IDataSeries input, double coeff, int cycle, double filter, int period, double shift, double splitTick)
        {
            return _indicator.NonLagDot(input, coeff, cycle, filter, period, shift, splitTick);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Conversion of the nonlagma v4 indicator from MetaTrader
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.NonLagDot NonLagDot(double coeff, int cycle, double filter, int period, double shift, double splitTick)
        {
            return _indicator.NonLagDot(Input, coeff, cycle, filter, period, shift, splitTick);
        }

        /// <summary>
        /// Conversion of the nonlagma v4 indicator from MetaTrader
        /// </summary>
        /// <returns></returns>
        public Indicator.NonLagDot NonLagDot(Data.IDataSeries input, double coeff, int cycle, double filter, int period, double shift, double splitTick)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.NonLagDot(input, coeff, cycle, filter, period, shift, splitTick);
        }
    }
}
#endregion
